# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Amazon Software License (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
# http://aws.amazon.com/asl/
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

<#
    .SYNOPSIS
    Retrieves the user data property from metadata, and runs the script it contains.

    .PARAMETER Username
    (Discontinued) A string that contains the name of the local user that runs the script. The user must have administrator permissions.

    .PARAMETER Password
    (Discontinued) A string that contains the plaintext password for the local user that runs the script.

    .PARAMETER Credential
    A PSCredential object that contains the name and password of the local user that runs the script. The user must have administrator permissions.

    .PARAMETER OnlyUnregister
    Set this parameter to unregister the scheduled task and skip the user data script.

    .PARAMETER OnlyExecute
    Set this parameter to run the user data script inline. This doesn't modify the scheduled task.

    .PARAMETER FinalizeTelemetry
    (Discontinued) An optional script block to finalize telemetry before running the user data script.
#>
function Invoke-Userdata
{
  [Diagnostics.CodeAnalysis.SuppressMessageAttribute('PSAvoidUsingPlainTextForPassword', '', Justification='BackwardsCompatibility')]
  [Diagnostics.CodeAnalysis.SuppressMessageAttribute('PSAvoidUsingUsernameAndPasswordParams', '', Justification='BackwardsCompatibility')]
  [CmdletBinding(DefaultParameterSetName='UsernamePassword')]
  [OutputType([System.Boolean])]
  param(
    [Parameter(Mandatory = $false, Position = 0, ParameterSetName='UsernamePassword')]
    [string]$Username,

    [Parameter(Mandatory = $false, Position = 1, ParameterSetName='UsernamePassword')]
    [string]$Password,

    [Parameter(Mandatory = $false, Position = 0, ParameterSetName='Credential')]
    [PSCredential]$AdminCredential,

    [Parameter(Mandatory = $false)]
    [switch]$OnlyUnregister,

    [Parameter(Mandatory = $false)]
    [switch]$OnlyExecute,

    [Parameter(Mandatory = $false)]
    [switch]$FromPersist,

    [Diagnostics.CodeAnalysis.SuppressMessageAttribute('PSReviewUnusedParameter', '$FinalizeTelemetry', Justification = 'BackwardsCompatibility')]
    [Parameter(Mandatory = $false)]
    $FinalizeTelemetry
  )

  $scheduleName = "Userdata Execution"

  if ($OnlyUnregister)
  {
    Register-FunctionScheduler -Function $MyInvocation.MyCommand -ScheduleName $scheduleName -Unregister
    return $null
  }

  if ($PSCmdlet.ParameterSetName -eq 'UsernamePassword')
  {
    Write-Log 'The Username and Password parameters are discontinued. Consider moving to the Credential parameter.'
    Write-Verbose -Message 'The Username and Password parameters are discontinued. Consider moving to the Credential parameter.'

    $AdminCredential = GetPSCredential -Username $Username -Password $Password

    Remove-Variable -Name Password
    [System.GC]::Collect()
  }

  $windowsTemp = Join-Path -Path $env:SystemRoot -ChildPath 'Temp'

  $handleUserDataState = Get-LaunchConfig -Key HandleUserData
  [System.GC]::Collect()
  if (!$handleUserDataState)
  {
    Write-Log "Handle user data is not enabled."
    return $false
  }

  # Sets the log file name to initialize it before calling any functions.
  Initialize-Log -FileName "UserdataExecution.log" -RestrictToAdmins $true

  try
  {
    Write-Log "Begin user data script process."

    $regexFormat = "(?is){0}(.*?){1}"

    $powershellContent = ""
    $powershellArgs = ""
    $batchContent = ""

    $fileLocation = Join-Path $env:LOCALAPPDATA -ChildPath "Temp\Amazon\EC2-Windows\Launch\InvokeUserData"
    if ($null -eq $fileLocation)
    {
      throw "Failed to create a path to invoke user data."
    }

    New-Item -Item Directory $fileLocation -Force

    # Add full control for the Administrators and LocalSystem groups, and the current user.
    $ACL = Get-Acl -Path $fileLocation
    $LocalSystem = New-Object System.Security.Principal.SecurityIdentifier 'S-1-5-18'
    $AllowLocalSystemFullControl = New-Object System.Security.AccessControl.FileSystemAccessRule (
      $LocalSystem,
      [System.Security.AccessControl.FileSystemRights]::FullControl,
      ([System.Security.AccessControl.InheritanceFlags]::ContainerInherit -bor [System.Security.AccessControl.InheritanceFlags]::ObjectInherit),
      [System.Security.AccessControl.PropagationFlags]::None,
      [System.Security.AccessControl.AccessControlType]::Allow
    )

    $AdministratorsGroup = New-Object System.Security.Principal.SecurityIdentifier 'S-1-5-32-544'
    $AllowAdministratorsFullControl = New-Object System.Security.AccessControl.FileSystemAccessRule (
      $AdministratorsGroup,
      [System.Security.AccessControl.FileSystemRights]::FullControl,
      ([System.Security.AccessControl.InheritanceFlags]::ContainerInherit -bor [System.Security.AccessControl.InheritanceFlags]::ObjectInherit),
      [System.Security.AccessControl.PropagationFlags]::None,
      [System.Security.AccessControl.AccessControlType]::Allow
    )

    $CurrentUser = [System.Security.Principal.WindowsIdentity]::GetCurrent().User
    $AllowCurrentUserFullControl = New-Object System.Security.AccessControl.FileSystemAccessRule (
      $CurrentUser,
      [System.Security.AccessControl.FileSystemRights]::FullControl,
      ([System.Security.AccessControl.InheritanceFlags]::ContainerInherit -bor [System.Security.AccessControl.InheritanceFlags]::ObjectInherit),
      [System.Security.AccessControl.PropagationFlags]::None,
      [System.Security.AccessControl.AccessControlType]::Allow
    )

    $ACL.AddAccessRule($AllowLocalSystemFullControl)
    $ACL.AddAccessRule($AllowAdministratorsFullControl)
    $ACL.AddAccessRule($AllowCurrentUserFullControl)
    (Get-Item $fileLocation).SetAccessControl($ACL)

    # Remove inheritance and don't keep inherited permissions
    $ACL = Get-Acl -Path $fileLocation
    $ACL.SetAccessRuleProtection($true,$false)
    (Get-Item $fileLocation).SetAccessControl($ACL)

    $userdata = Get-Metadata -UrlFragment "user-data"
    if (-not $userdata)
    {
      # If there is no user data for this launch, unregister any associated scheduled task.
      Register-FunctionScheduler -Function $MyInvocation.MyCommand -ScheduleName $scheduleName -Unregister
      Write-Log "Launch metadata did not include a user data script."
      return $null
    }

    $userdataContent = $userdata.Trim()

    # The user data script runs as the local administrator by default. However, if the password is empty, then user data
    # runs as the local system.
    try
    {
        $runAsLocalSystem = -not $AdminCredential.UserName -or -not $AdminCredential.GetNetworkCredential().Password
    }
    catch [System.Management.Automation.RuntimeException]
    {
        # A runtime exception occurs if the Credential object is null.
        $runAsLocalSystem = $true
    }
    [System.GC]::Collect()
    $persist = $false

    # If user data contains a <persist> tag set with a value of 'true' and the launch agent is scheduled to only run on
    # the first boot, schedule a task to ensure that user data runs on every boot.
    $persistRegex = [regex]($regexFormat -f "<persist>","</persist>")
    $persistMatch = $persistRegex.Matches($userdataContent)
    if ($persistMatch.Success -and $persistMatch.Captures.Count -eq 1 -and $persistMatch.Groups.Count -eq 2)
    {
      $persistValue = $persistMatch.Groups[1].Value
      Write-Log ("User data contains the <persist> tag." -f $persistValue)
      if ($persistValue -ieq "true")
      {
        Write-Log "Running user data on every boot"
        $persist = $true
      }
    }
    else
    {
      Write-Log "Unable to parse <persist> tags. This can happen when tags are unmatched or poorly formed."
    }

    if ($OnlyExecute)
    {
      Write-Log ("Run user data script inline, and not as a separate task. Persist value: {0}." -f $persist)
    }
    elseif ($persist)
    {
      Register-FunctionScheduler -Function $MyInvocation.MyCommand -Arguments "-FromPersist" -ScheduleName $scheduleName
    }
    else
    {
      Write-Log "Unregister the scheduled task to persist user data."
      Register-FunctionScheduler -Function $MyInvocation.MyCommand -ScheduleName $scheduleName -Unregister
      if ($FromPersist)
      {
        # If the function was called from scheduled task and persist tag is not found, don't run it at all.
        return $persist
      }
    }

    # Parse runAsLocalSystem from user data.
    $runAsLocalSystemRegex = [regex]($regexFormat -f "<runAsLocalSystem>","</runAsLocalSystem>")
    $runAsLocalSystemMatch = $runAsLocalSystemRegex.Matches($userdataContent)
    if ($runAsLocalSystemMatch.Success -and $runAsLocalSystemMatch.Captures.Count -eq 1 -and $runAsLocalSystemMatch.Groups.Count -eq 2)
    {
      $runAsLocalSystemValue = $runAsLocalSystemMatch.Groups[1].Value
      Write-Log ("<runAsLocalSystem> tag was provided: {0}" -f $runAsLocalSystemValue)
      if ($runAsLocalSystemValue -ieq "true")
      {
        Write-Log "Running user data as local system."
        $runAsLocalSystem = $true
      }
    }
    else
    {
      Write-Log "Unable to parse <runAsLocalSystem> tags. This can happen when tags are unmatched or poorly formed."
    }

    # Parse script from user data.
    $scriptRegex = [regex]($regexFormat -f "<script>","</script>")
    $scriptMatch = $scriptRegex.Matches($userdataContent)
    if ($scriptMatch.Success -and $scriptMatch.Captures.Count -eq 1)
    {
      $batchContent = $scriptMatch.Groups[1].Value
    }
    else
    {
      Write-Log "Unable to parse <script> tags. This can happen when tags are unmatched or poorly formed."
    }

    # Parse PowerShell script from user data.
    $powershellRegex = [regex]($regexFormat -f "<powershell>","</powershell>")
    $powershellMatch = $powershellRegex.Matches($userdataContent)
    if ($powershellMatch.Success -and $powershellMatch.Captures.Count -eq 1)
    {
      $powershellContent = $powershellMatch.Groups[1].Value
    }
    else
    {
      Write-Log "Unable to parse <powershell> tags. This can happen when tags are unmatched or poorly formed."
    }

    # Parse PowerShell arguments from user data.
    $powershellArgsRegex = [regex]($regexFormat -f "<powershellArguments>","</powershellArguments>")
    $powershellArgsMatch = $powershellArgsRegex.Matches($userdataContent)
    if ($powershellArgsMatch.Success -and $powershellArgsMatch.Captures.Count -eq 1)
    {
      $powershellArgs = $powershellArgsMatch.Groups[1].Value
    }
    else
    {
      Write-Log "Unable to parse <powershellArguments> tags. This can happen when tags are unmatched or poorly formed."
    }

    # Run batch commands first.
    if ($batchContent)
    {
      Write-Log "<script> tag was provided.. running script content"

      $errorFile = Join-Path $fileLocation -ChildPath "InvokeUserdataErrors.log"
      $outputFile = Join-Path $fileLocation -ChildPath "InvokeUserdataOutput.log"

      $filePath = "$env:LOCALAPPDATA\Temp\Amazon\EC2-Windows\Launch\InvokeUserData\UserScript.bat"
      $batchContent | Out-File $filePath -Encoding ascii

      if ($runAsLocalSystem)
      {
        Start-Process $script:cmdPath -ArgumentList "/C",`"$filePath`" -Wait -NoNewWindow -RedirectStandardOutput $outputFile -RedirectStandardError $errorFile
      }
      else
      {
        Invoke-CmdAsAdmin -AdminCredential $AdminCredential -Command "$script:cmdPath /C `"`"$filePath`" 1> `"$outputFile`" 2> `"$errorFile`"`""
      }

      try
      {
        # Save a copy of the UserScript.bat file, along with the errorFile and outputFile to the system Temp folder for backwards compatibility.
        Copy-Item -Path "$errorFile" -Destination $windowsTemp -ErrorAction Stop
        Copy-Item -Path "$outputFile" -Destination $windowsTemp -ErrorAction Stop
        Copy-Item -Path "$filePath" -Destination $windowsTemp -ErrorAction Stop
      }
      catch
      {
        Write-ErrorLog ("Failed to save Batch user-data files to the Windows temp folder: {0}" -f $_.Exception.Message)
      }

      $output = Get-Content $errorFile -Raw
      if ($output)
      {
        Write-Log ("Message: The errors from user data script: {0}" -f $output)
      }

      $output = Get-Content $outputFile -Raw
      if ($output)
      {
        Write-Log ("Message: The output from user data script: {0}" -f $output)
      }
    }

    # Run PowerShell commands.
    if ($powershellContent)
    {
      Write-Log "<powershell> tag was provided.. running powershell content"

      $errorFile = Join-Path $fileLocation -ChildPath "InvokeUserdataErrors.log"
      $outputFile = Join-Path $fileLocation -ChildPath "InvokeUserdataOutput.log"

      $filePath = "$env:LOCALAPPDATA\Temp\Amazon\EC2-Windows\Launch\InvokeUserData\UserScript.ps1"
      $powershellContent | Out-File $filePath

      if (-not $powershellArgs)
      {
        # Use the arguments provided. If no arguments are provided, default to set the execution policy to
        # Unrestricted.
        $powershellArgs = "-ExecutionPolicy Unrestricted"
      }

      if ($runAsLocalSystem)
      {
        Start-Process $script:psPath -ArgumentList $powershellArgs,".",`'$filePath`' -Wait -NoNewWindow -RedirectStandardOutput $outputFile -RedirectStandardError $errorFile
      }
      else
      {
        # Get the credentials for the local Administrator account to run the user data script with elevated permissions.
        $user = Get-CimInstance -ClassName Win32_UserAccount -Filter "LocalAccount = 'True'" | Where-Object { $_.SID -like 'S-1-5-21-*' -and $_.SID -like '*-500' }
        # The PSCredential constructor accepts a user name as String and a password as SecureString. The password usage
        # here is a SecureString.
        $tempCredential = New-Object -TypeName 'System.Management.Automation.PSCredential' -ArgumentList $user.Name, $AdminCredential.Password
        Invoke-CmdAsAdmin -AdminCredential $tempCredential -Command "$script:cmdPath /C `"$script:psPath $powershellArgs . `'$filePath`' 1> `"$outputFile`" 2> `"$errorFile`"`""
      }

      try
      {
        # Save a copy of the UserScript.ps1 file, along with the errorFile and outputFile to the system Temp folder to
        # preserve backwards compatibility.
        Copy-Item -Path "$errorFile" -Destination $windowsTemp -ErrorAction Stop
        Copy-Item -Path "$outputFile" -Destination $windowsTemp -ErrorAction Stop
        Copy-Item -Path "$filePath" -Destination $windowsTemp -ErrorAction Stop
      }
      catch
      {
        Write-ErrorLog ("Failed to save PowerShell user-data files to the Windows temp folder: {0}" -f $_.Exception.Message)
      }

      $output = Get-Content $errorFile -Raw
      if ($output)
      {
        Write-Log ("Message: The errors from user data script: {0}" -f $output)
      }

      $output = Get-Content $outputFile -Raw
      if ($output)
      {
        Write-Log ("Message: The output from user data script: {0}" -f $output)
      }
    }
  }
  catch
  {
    Write-ErrorLog ("Unable to run user data script, due to the following exception: {0}" -f $_.Exception.Message)
  }
  finally
  {
    Write-Log "User data script completed."

    # Complete the log before sending the return value.
    Complete-Log
  }

  return $persist
}

# SIG # Begin signature block
# MIIuegYJKoZIhvcNAQcCoIIuazCCLmcCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCCbWf00+g1RIC7L
# JSa7YlrdC94Sj60meR8VfwY6vgQdX6CCE+YwggXAMIIEqKADAgECAhAP0bvKeWvX
# +N1MguEKmpYxMA0GCSqGSIb3DQEBCwUAMGwxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xKzApBgNV
# BAMTIkRpZ2lDZXJ0IEhpZ2ggQXNzdXJhbmNlIEVWIFJvb3QgQ0EwHhcNMjIwMTEz
# MDAwMDAwWhcNMzExMTA5MjM1OTU5WjBiMQswCQYDVQQGEwJVUzEVMBMGA1UEChMM
# RGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSEwHwYDVQQD
# ExhEaWdpQ2VydCBUcnVzdGVkIFJvb3QgRzQwggIiMA0GCSqGSIb3DQEBAQUAA4IC
# DwAwggIKAoICAQC/5pBzaN675F1KPDAiMGkz7MKnJS7JIT3yithZwuEppz1Yq3aa
# za57G4QNxDAf8xukOBbrVsaXbR2rsnnyyhHS5F/WBTxSD1Ifxp4VpX6+n6lXFllV
# cq9ok3DCsrp1mWpzMpTREEQQLt+C8weE5nQ7bXHiLQwb7iDVySAdYyktzuxeTsiT
# +CFhmzTrBcZe7FsavOvJz82sNEBfsXpm7nfISKhmV1efVFiODCu3T6cw2Vbuyntd
# 463JT17lNecxy9qTXtyOj4DatpGYQJB5w3jHtrHEtWoYOAMQjdjUN6QuBX2I9YI+
# EJFwq1WCQTLX2wRzKm6RAXwhTNS8rhsDdV14Ztk6MUSaM0C/CNdaSaTC5qmgZ92k
# J7yhTzm1EVgX9yRcRo9k98FpiHaYdj1ZXUJ2h4mXaXpI8OCiEhtmmnTK3kse5w5j
# rubU75KSOp493ADkRSWJtppEGSt+wJS00mFt6zPZxd9LBADMfRyVw4/3IbKyEbe7
# f/LVjHAsQWCqsWMYRJUadmJ+9oCw++hkpjPRiQfhvbfmQ6QYuKZ3AeEPlAwhHbJU
# KSWJbOUOUlFHdL4mrLZBdd56rF+NP8m800ERElvlEFDrMcXKchYiCd98THU/Y+wh
# X8QgUWtvsauGi0/C1kVfnSD8oR7FwI+isX4KJpn15GkvmB0t9dmpsh3lGwIDAQAB
# o4IBZjCCAWIwDwYDVR0TAQH/BAUwAwEB/zAdBgNVHQ4EFgQU7NfjgtJxXWRM3y5n
# P+e6mK4cD08wHwYDVR0jBBgwFoAUsT7DaQP4v0cB1JgmGggC72NkK8MwDgYDVR0P
# AQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMH8GCCsGAQUFBwEBBHMwcTAk
# BggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEkGCCsGAQUFBzAC
# hj1odHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRIaWdoQXNzdXJh
# bmNlRVZSb290Q0EuY3J0MEsGA1UdHwREMEIwQKA+oDyGOmh0dHA6Ly9jcmwzLmRp
# Z2ljZXJ0LmNvbS9EaWdpQ2VydEhpZ2hBc3N1cmFuY2VFVlJvb3RDQS5jcmwwHAYD
# VR0gBBUwEzAHBgVngQwBAzAIBgZngQwBBAEwDQYJKoZIhvcNAQELBQADggEBAEHx
# qRH0DxNHecllao3A7pgEpMbjDPKisedfYk/ak1k2zfIe4R7sD+EbP5HU5A/C5pg0
# /xkPZigfT2IxpCrhKhO61z7H0ZL+q93fqpgzRh9Onr3g7QdG64AupP2uU7SkwaT1
# IY1rzAGt9Rnu15ClMlIr28xzDxj4+87eg3Gn77tRWwR2L62t0+od/P1Tk+WMieNg
# GbngLyOOLFxJy34riDkruQZhiPOuAnZ2dMFkkbiJUZflhX0901emWG4f7vtpYeJa
# 3Cgh6GO6Ps9W7Zrk9wXqyvPsEt84zdp7PiuTUy9cUQBY3pBIowrHC/Q7bVUx8ALM
# R3eWUaNetbxcyEMRoacwggawMIIEmKADAgECAhAIrUCyYNKcTJ9ezam9k67ZMA0G
# CSqGSIb3DQEBDAUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJ
# bmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0
# IFRydXN0ZWQgUm9vdCBHNDAeFw0yMTA0MjkwMDAwMDBaFw0zNjA0MjgyMzU5NTla
# MGkxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UE
# AxM4RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEz
# ODQgMjAyMSBDQTEwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDVtC9C
# 0CiteLdd1TlZG7GIQvUzjOs9gZdwxbvEhSYwn6SOaNhc9es0JAfhS0/TeEP0F9ce
# 2vnS1WcaUk8OoVf8iJnBkcyBAz5NcCRks43iCH00fUyAVxJrQ5qZ8sU7H/Lvy0da
# E6ZMswEgJfMQ04uy+wjwiuCdCcBlp/qYgEk1hz1RGeiQIXhFLqGfLOEYwhrMxe6T
# SXBCMo/7xuoc82VokaJNTIIRSFJo3hC9FFdd6BgTZcV/sk+FLEikVoQ11vkunKoA
# FdE3/hoGlMJ8yOobMubKwvSnowMOdKWvObarYBLj6Na59zHh3K3kGKDYwSNHR7Oh
# D26jq22YBoMbt2pnLdK9RBqSEIGPsDsJ18ebMlrC/2pgVItJwZPt4bRc4G/rJvmM
# 1bL5OBDm6s6R9b7T+2+TYTRcvJNFKIM2KmYoX7BzzosmJQayg9Rc9hUZTO1i4F4z
# 8ujo7AqnsAMrkbI2eb73rQgedaZlzLvjSFDzd5Ea/ttQokbIYViY9XwCFjyDKK05
# huzUtw1T0PhH5nUwjewwk3YUpltLXXRhTT8SkXbev1jLchApQfDVxW0mdmgRQRNY
# mtwmKwH0iU1Z23jPgUo+QEdfyYFQc4UQIyFZYIpkVMHMIRroOBl8ZhzNeDhFMJlP
# /2NPTLuqDQhTQXxYPUez+rbsjDIJAsxsPAxWEQIDAQABo4IBWTCCAVUwEgYDVR0T
# AQH/BAgwBgEB/wIBADAdBgNVHQ4EFgQUaDfg67Y7+F8Rhvv+YXsIiGX0TkIwHwYD
# VR0jBBgwFoAU7NfjgtJxXWRM3y5nP+e6mK4cD08wDgYDVR0PAQH/BAQDAgGGMBMG
# A1UdJQQMMAoGCCsGAQUFBwMDMHcGCCsGAQUFBwEBBGswaTAkBggrBgEFBQcwAYYY
# aHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEEGCCsGAQUFBzAChjVodHRwOi8vY2Fj
# ZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNydDBDBgNV
# HR8EPDA6MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRU
# cnVzdGVkUm9vdEc0LmNybDAcBgNVHSAEFTATMAcGBWeBDAEDMAgGBmeBDAEEATAN
# BgkqhkiG9w0BAQwFAAOCAgEAOiNEPY0Idu6PvDqZ01bgAhql+Eg08yy25nRm95Ry
# sQDKr2wwJxMSnpBEn0v9nqN8JtU3vDpdSG2V1T9J9Ce7FoFFUP2cvbaF4HZ+N3HL
# IvdaqpDP9ZNq4+sg0dVQeYiaiorBtr2hSBh+3NiAGhEZGM1hmYFW9snjdufE5Btf
# Q/g+lP92OT2e1JnPSt0o618moZVYSNUa/tcnP/2Q0XaG3RywYFzzDaju4ImhvTnh
# OE7abrs2nfvlIVNaw8rpavGiPttDuDPITzgUkpn13c5UbdldAhQfQDN8A+KVssIh
# dXNSy0bYxDQcoqVLjc1vdjcshT8azibpGL6QB7BDf5WIIIJw8MzK7/0pNVwfiThV
# 9zeKiwmhywvpMRr/LhlcOXHhvpynCgbWJme3kuZOX956rEnPLqR0kq3bPKSchh/j
# wVYbKyP/j7XqiHtwa+aguv06P0WmxOgWkVKLQcBIhEuWTatEQOON8BUozu3xGFYH
# Ki8QxAwIZDwzj64ojDzLj4gLDb879M4ee47vtevLt/B3E+bnKD+sEq6lLyJsQfmC
# XBVmzGwOysWGw/YmMwwHS6DTBwJqakAwSEs0qFEgu60bhQjiWQ1tygVQK+pKHJ6l
# /aCnHwZ05/LWUpD9r4VIIflXO7ScA+2GRfS0YW6/aOImYIbqyK+p/pQd52MbOoZW
# eE4wggdqMIIFUqADAgECAhAPo79ZA+boHMkRkUNkUoBpMA0GCSqGSIb3DQEBCwUA
# MGkxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UE
# AxM4RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEz
# ODQgMjAyMSBDQTEwHhcNMjMwOTIxMDAwMDAwWhcNMjQwOTIwMjM1OTU5WjCB8jET
# MBEGCysGAQQBgjc8AgEDEwJVUzEZMBcGCysGAQQBgjc8AgECEwhEZWxhd2FyZTEd
# MBsGA1UEDwwUUHJpdmF0ZSBPcmdhbml6YXRpb24xEDAOBgNVBAUTBzQxNTI5NTQx
# CzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdTZWF0
# dGxlMSIwIAYDVQQKExlBbWF6b24gV2ViIFNlcnZpY2VzLCBJbmMuMRMwEQYDVQQL
# EwpBbWF6b24gRUMyMSIwIAYDVQQDExlBbWF6b24gV2ViIFNlcnZpY2VzLCBJbmMu
# MIIBojANBgkqhkiG9w0BAQEFAAOCAY8AMIIBigKCAYEAyQgSqeEEf92N/lmb69Tz
# t7Qg0dCGMeN5rpoKPFsW6OpN0p/9gNqeqHTGjeKITMHC2F2OkQ+adZfFIniOKVuB
# kzAgr1JvLGP7nGCB1qfU0DowEG5YOQYxq0+ez8ARXA5vWO2DkJyJ72upyEBV+Uuc
# yffwPV8iino2w2JcKBgSBwrfK09odd3TvacQrGZflbOvBZkRwzpgbcqKadU+xWlc
# absOS/jZhk+lXpEFPx/6U2PA8F+FpTFHqrrsB8l7LHqqdPHfB6CwszM16KnpWc07
# vI7ESveeKHWEXra3Fg70XmOby8bUTlW9U2IfotsH4yxWy/odA1nFBZ7ihXFN+IhK
# ABRFlAMHLjhd/ZzmexZH0ZAfPSrPUwfeMAJNKX8lHnuz5gQT0ukhhXyyp+JY2X9q
# ii6BopteCUYMEj/4KPQuTHbYSK+HVDk8yUpt7ZnFkihJ4eAQ+doHuMPufJ+rnxOs
# a6UahM1ts+ili7Nym8j1cuzQjfrjjysjRxqUGI1+9FVFAgMBAAGjggICMIIB/jAf
# BgNVHSMEGDAWgBRoN+Drtjv4XxGG+/5hewiIZfROQjAdBgNVHQ4EFgQUN8tpbyMu
# Fv5oH1HZXuepuLWCBrEwPQYDVR0gBDYwNDAyBgVngQwBAzApMCcGCCsGAQUFBwIB
# FhtodHRwOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMwDgYDVR0PAQH/BAQDAgeAMBMG
# A1UdJQQMMAoGCCsGAQUFBwMDMIG1BgNVHR8Ega0wgaowU6BRoE+GTWh0dHA6Ly9j
# cmwzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNENvZGVTaWduaW5nUlNB
# NDA5NlNIQTM4NDIwMjFDQTEuY3JsMFOgUaBPhk1odHRwOi8vY3JsNC5kaWdpY2Vy
# dC5jb20vRGlnaUNlcnRUcnVzdGVkRzRDb2RlU2lnbmluZ1JTQTQwOTZTSEEzODQy
# MDIxQ0ExLmNybDCBlAYIKwYBBQUHAQEEgYcwgYQwJAYIKwYBBQUHMAGGGGh0dHA6
# Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBcBggrBgEFBQcwAoZQaHR0cDovL2NhY2VydHMu
# ZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZEc0Q29kZVNpZ25pbmdSU0E0MDk2
# U0hBMzg0MjAyMUNBMS5jcnQwCQYDVR0TBAIwADANBgkqhkiG9w0BAQsFAAOCAgEA
# WtvDAAUi8Az7ODQJNSv4R7zXjN8JRghkMSQHD8CIcAuLER+B7zfbHLqwU8sdzxQs
# VkMGAPlwrcybnqFipQgOpz3MUHiAseVzWRz/7RIkmZDL6c6X4wJka1si5P+XxsC1
# KGFbYgIu+n4sOf00AOHuSYYtiE85K0JbClplz2Oc1E8sb4V2Xpxg7vzPLuCInD84
# E6TFcmIZv32RiLcgWNv+lXlWr4KCu4kvqKWuB98zxlyIVvUTyGRJMBeOKYXCxPUN
# xvVQ33g2fl1YYmCAl4u+0Thlw/LnibUNsUO08uKYqqynCjw7GNA9q0syTOe1eB6a
# d8y3E6zA2kVVRGouj43MWdniAheQ9HUsSQ8uNK6IrhPDU75DxB+UTGCAc4Y8ZYbE
# 3OtazqEMzH3kBU9o2JLZ/EE+0Cs0qlFT0xFQvEGn8VnEaQ+V0RfUwU8XJQUbkh20
# edcdODciyg3BUTqP18IcW16lS7Bvjmy/I+9mHwDZKN5Lxs42Rs4+NY7YxK+Uau8F
# lx7s1jibMa0g15HJFvmEO4BB9r4jozQykGb30E7smq+lvsMFLlv4L956TXfJngZI
# 3P7HAaUORR+nieq2ujgqpBiAwccIcK+YtRsoCBNp0C0AMdWEeSVLf6mAE61JVxju
# N+FxDlvDx5Y0tszY58Uw+N4jwVT8Z1D6q1lG5nmYPsYxghnqMIIZ5gIBATB9MGkx
# CzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UEAxM4
# RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEzODQg
# MjAyMSBDQTECEA+jv1kD5ugcyRGRQ2RSgGkwDQYJYIZIAWUDBAIBBQCgfDAQBgor
# BgEEAYI3AgEMMQIwADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEE
# AYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0BCQQxIgQgMN9uQewscZkS
# 9nsyI44fYwM20v3C/ZacGmopd1cqNe0wDQYJKoZIhvcNAQEBBQAEggGAOEvMMpdT
# SJvbwowyr/cXe7vo1MMEGcunKuX/1duFDxEyB6kr76tk5hT6omWa7Yg/O0Jd4WbK
# AYqZSiyoN3aLXq8Lt/SZxkyyxWQboF1PQ4alls8laL/qamtiBdiQ1td+RZiLFhIW
# 4lcd7Nje26+lKKMbBT5VdKm11Ugtua84ui+U9qwDfQ6CNfQDonXu8UQD/d7cfOX1
# GnkG79S4/HC2gj+kjFF3oKXDLUPlXQk2l7pxRo/A7Q+4SRGjtqWA7x/u2iJ73WCy
# AD3bSLSJIZUkUh3XN9mKTAH+Ynzhfu2wa2QXYkCIoa2DE584bwiW3bxvgt9d2WBn
# rLS+TGe3zeL8/PooM7XBFCVLihp/4sQZAmnlkBOcvu6VsVilCDaKhRfOCGkJW4kr
# +3NQiUJ7KzPiHCSasnT/IdnTPvSUvZaemfC1/lyczobfv0VXg2j2T1lnk8rBq8BO
# I6evP+MfMqOKH6T3L6IiJB9+kHEsPMdEdwTR+OQt23Dr8L4SqIrBQ4H7oYIXQDCC
# FzwGCisGAQQBgjcDAwExghcsMIIXKAYJKoZIhvcNAQcCoIIXGTCCFxUCAQMxDzAN
# BglghkgBZQMEAgEFADB4BgsqhkiG9w0BCRABBKBpBGcwZQIBAQYJYIZIAYb9bAcB
# MDEwDQYJYIZIAWUDBAIBBQAEIGuKgJITpeQSxkUgeXfL7S/5xsTul+iejEqzIQQK
# adaoAhEA4psa0aSOEPbm8DSc++RtLRgPMjAyNDAxMTYwMTM2NTBaoIITCTCCBsIw
# ggSqoAMCAQICEAVEr/OUnQg5pr/bP1/lYRYwDQYJKoZIhvcNAQELBQAwYzELMAkG
# A1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMTswOQYDVQQDEzJEaWdp
# Q2VydCBUcnVzdGVkIEc0IFJTQTQwOTYgU0hBMjU2IFRpbWVTdGFtcGluZyBDQTAe
# Fw0yMzA3MTQwMDAwMDBaFw0zNDEwMTMyMzU5NTlaMEgxCzAJBgNVBAYTAlVTMRcw
# FQYDVQQKEw5EaWdpQ2VydCwgSW5jLjEgMB4GA1UEAxMXRGlnaUNlcnQgVGltZXN0
# YW1wIDIwMjMwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQCjU0WHHYOO
# W6w+VLMj4M+f1+XS512hDgncL0ijl3o7Kpxn3GIVWMGpkxGnzaqyat0QKYoeYmNp
# 01icNXG/OpfrlFCPHCDqx5o7L5Zm42nnaf5bw9YrIBzBl5S0pVCB8s/LB6YwaMqD
# Qtr8fwkklKSCGtpqutg7yl3eGRiF+0XqDWFsnf5xXsQGmjzwxS55DxtmUuPI1j5f
# 2kPThPXQx/ZILV5FdZZ1/t0QoRuDwbjmUpW1R9d4KTlr4HhZl+NEK0rVlc7vCBfq
# gmRN/yPjyobutKQhZHDr1eWg2mOzLukF7qr2JPUdvJscsrdf3/Dudn0xmWVHVZ1K
# JC+sK5e+n+T9e3M+Mu5SNPvUu+vUoCw0m+PebmQZBzcBkQ8ctVHNqkxmg4hoYru8
# QRt4GW3k2Q/gWEH72LEs4VGvtK0VBhTqYggT02kefGRNnQ/fztFejKqrUBXJs8q8
# 18Q7aESjpTtC/XN97t0K/3k0EH6mXApYTAA+hWl1x4Nk1nXNjxJ2VqUk+tfEayG6
# 6B80mC866msBsPf7Kobse1I4qZgJoXGybHGvPrhvltXhEBP+YUcKjP7wtsfVx95s
# JPC/QoLKoHE9nJKTBLRpcCcNT7e1NtHJXwikcKPsCvERLmTgyyIryvEoEyFJUX4G
# ZtM7vvrrkTjYUQfKlLfiUKHzOtOKg8tAewIDAQABo4IBizCCAYcwDgYDVR0PAQH/
# BAQDAgeAMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwIAYD
# VR0gBBkwFzAIBgZngQwBBAIwCwYJYIZIAYb9bAcBMB8GA1UdIwQYMBaAFLoW2W1N
# hS9zKXaaL3WMaiCPnshvMB0GA1UdDgQWBBSltu8T5+/N0GSh1VapZTGj3tXjSTBa
# BgNVHR8EUzBRME+gTaBLhklodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNl
# cnRUcnVzdGVkRzRSU0E0MDk2U0hBMjU2VGltZVN0YW1waW5nQ0EuY3JsMIGQBggr
# BgEFBQcBAQSBgzCBgDAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQu
# Y29tMFgGCCsGAQUFBzAChkxodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGln
# aUNlcnRUcnVzdGVkRzRSU0E0MDk2U0hBMjU2VGltZVN0YW1waW5nQ0EuY3J0MA0G
# CSqGSIb3DQEBCwUAA4ICAQCBGtbeoKm1mBe8cI1PijxonNgl/8ss5M3qXSKS7Iwi
# Aqm4z4Co2efjxe0mgopxLxjdTrbebNfhYJwr7e09SI64a7p8Xb3CYTdoSXej65Cq
# EtcnhfOOHpLawkA4n13IoC4leCWdKgV6hCmYtld5j9smViuw86e9NwzYmHZPVrlS
# wradOKmB521BXIxp0bkrxMZ7z5z6eOKTGnaiaXXTUOREEr4gDZ6pRND45Ul3CFoh
# xbTPmJUaVLq5vMFpGbrPFvKDNzRusEEm3d5al08zjdSNd311RaGlWCZqA0Xe2VC1
# UIyvVr1MxeFGxSjTredDAHDezJieGYkD6tSRN+9NUvPJYCHEVkft2hFLjDLDiOZY
# 4rbbPvlfsELWj+MXkdGqwFXjhr+sJyxB0JozSqg21Llyln6XeThIX8rC3D0y33XW
# Nmdaifj2p8flTzU8AL2+nCpseQHc2kTmOt44OwdeOVj0fHMxVaCAEcsUDH6uvP6k
# 63llqmjWIso765qCNVcoFstp8jKastLYOrixRoZruhf9xHdsFWyuq69zOuhJRrfV
# f8y2OMDY7Bz1tqG4QyzfTkx9HmhwwHcK1ALgXGC7KP845VJa1qwXIiNO9OzTF/tQ
# a/8Hdx9xl0RBybhG02wyfFgvZ0dl5Rtztpn5aywGRu9BHvDwX+Db2a2QgESvgBBB
# ijCCBq4wggSWoAMCAQICEAc2N7ckVHzYR6z9KGYqXlswDQYJKoZIhvcNAQELBQAw
# YjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQ
# d3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UEAxMYRGlnaUNlcnQgVHJ1c3RlZCBSb290
# IEc0MB4XDTIyMDMyMzAwMDAwMFoXDTM3MDMyMjIzNTk1OVowYzELMAkGA1UEBhMC
# VVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMTswOQYDVQQDEzJEaWdpQ2VydCBU
# cnVzdGVkIEc0IFJTQTQwOTYgU0hBMjU2IFRpbWVTdGFtcGluZyBDQTCCAiIwDQYJ
# KoZIhvcNAQEBBQADggIPADCCAgoCggIBAMaGNQZJs8E9cklRVcclA8TykTepl1Gh
# 1tKD0Z5Mom2gsMyD+Vr2EaFEFUJfpIjzaPp985yJC3+dH54PMx9QEwsmc5Zt+Feo
# An39Q7SE2hHxc7Gz7iuAhIoiGN/r2j3EF3+rGSs+QtxnjupRPfDWVtTnKC3r07G1
# decfBmWNlCnT2exp39mQh0YAe9tEQYncfGpXevA3eZ9drMvohGS0UvJ2R/dhgxnd
# X7RUCyFobjchu0CsX7LeSn3O9TkSZ+8OpWNs5KbFHc02DVzV5huowWR0QKfAcsW6
# Th+xtVhNef7Xj3OTrCw54qVI1vCwMROpVymWJy71h6aPTnYVVSZwmCZ/oBpHIEPj
# Q2OAe3VuJyWQmDo4EbP29p7mO1vsgd4iFNmCKseSv6De4z6ic/rnH1pslPJSlREr
# WHRAKKtzQ87fSqEcazjFKfPKqpZzQmiftkaznTqj1QPgv/CiPMpC3BhIfxQ0z9JM
# q++bPf4OuGQq+nUoJEHtQr8FnGZJUlD0UfM2SU2LINIsVzV5K6jzRWC8I41Y99xh
# 3pP+OcD5sjClTNfpmEpYPtMDiP6zj9NeS3YSUZPJjAw7W4oiqMEmCPkUEBIDfV8j
# u2TjY+Cm4T72wnSyPx4JduyrXUZ14mCjWAkBKAAOhFTuzuldyF4wEr1GnrXTdrnS
# DmuZDNIztM2xAgMBAAGjggFdMIIBWTASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1Ud
# DgQWBBS6FtltTYUvcyl2mi91jGogj57IbzAfBgNVHSMEGDAWgBTs1+OC0nFdZEzf
# Lmc/57qYrhwPTzAOBgNVHQ8BAf8EBAMCAYYwEwYDVR0lBAwwCgYIKwYBBQUHAwgw
# dwYIKwYBBQUHAQEEazBpMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2Vy
# dC5jb20wQQYIKwYBBQUHMAKGNWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9E
# aWdpQ2VydFRydXN0ZWRSb290RzQuY3J0MEMGA1UdHwQ8MDowOKA2oDSGMmh0dHA6
# Ly9jcmwzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRSb290RzQuY3JsMCAG
# A1UdIAQZMBcwCAYGZ4EMAQQCMAsGCWCGSAGG/WwHATANBgkqhkiG9w0BAQsFAAOC
# AgEAfVmOwJO2b5ipRCIBfmbW2CFC4bAYLhBNE88wU86/GPvHUF3iSyn7cIoNqilp
# /GnBzx0H6T5gyNgL5Vxb122H+oQgJTQxZ822EpZvxFBMYh0MCIKoFr2pVs8Vc40B
# IiXOlWk/R3f7cnQU1/+rT4osequFzUNf7WC2qk+RZp4snuCKrOX9jLxkJodskr2d
# fNBwCnzvqLx1T7pa96kQsl3p/yhUifDVinF2ZdrM8HKjI/rAJ4JErpknG6skHibB
# t94q6/aesXmZgaNWhqsKRcnfxI2g55j7+6adcq/Ex8HBanHZxhOACcS2n82HhyS7
# T6NJuXdmkfFynOlLAlKnN36TU6w7HQhJD5TNOXrd/yVjmScsPT9rp/Fmw0HNT7ZA
# myEhQNC3EyTN3B14OuSereU0cZLXJmvkOHOrpgFPvT87eK1MrfvElXvtCl8zOYdB
# eHo46Zzh3SP9HSjTx/no8Zhf+yvYfvJGnXUsHicsJttvFXseGYs2uJPU5vIXmVnK
# cPA3v5gA3yAWTyf7YGcWoWa63VXAOimGsJigK+2VQbc61RWYMbRiCQ8KvYHZE/6/
# pNHzV9m8BPqC3jLfBInwAM1dwvnQI38AC+R2AibZ8GV2QqYphwlHK+Z/GqSFD/yY
# lvZVVCsfgPrA8g4r5db7qS9EFUrnEw4d2zc4GqEr9u3WfPwwggWNMIIEdaADAgEC
# AhAOmxiO+dAt5+/bUOIIQBhaMA0GCSqGSIb3DQEBDAUAMGUxCzAJBgNVBAYTAlVT
# MRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5j
# b20xJDAiBgNVBAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0yMjA4
# MDEwMDAwMDBaFw0zMTExMDkyMzU5NTlaMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNV
# BAMTGERpZ2lDZXJ0IFRydXN0ZWQgUm9vdCBHNDCCAiIwDQYJKoZIhvcNAQEBBQAD
# ggIPADCCAgoCggIBAL/mkHNo3rvkXUo8MCIwaTPswqclLskhPfKK2FnC4SmnPVir
# dprNrnsbhA3EMB/zG6Q4FutWxpdtHauyefLKEdLkX9YFPFIPUh/GnhWlfr6fqVcW
# WVVyr2iTcMKyunWZanMylNEQRBAu34LzB4TmdDttceItDBvuINXJIB1jKS3O7F5O
# yJP4IWGbNOsFxl7sWxq868nPzaw0QF+xembud8hIqGZXV59UWI4MK7dPpzDZVu7K
# e13jrclPXuU15zHL2pNe3I6PgNq2kZhAkHnDeMe2scS1ahg4AxCN2NQ3pC4FfYj1
# gj4QkXCrVYJBMtfbBHMqbpEBfCFM1LyuGwN1XXhm2ToxRJozQL8I11pJpMLmqaBn
# 3aQnvKFPObURWBf3JFxGj2T3wWmIdph2PVldQnaHiZdpekjw4KISG2aadMreSx7n
# DmOu5tTvkpI6nj3cAORFJYm2mkQZK37AlLTSYW3rM9nF30sEAMx9HJXDj/chsrIR
# t7t/8tWMcCxBYKqxYxhElRp2Yn72gLD76GSmM9GJB+G9t+ZDpBi4pncB4Q+UDCEd
# slQpJYls5Q5SUUd0viastkF13nqsX40/ybzTQRESW+UQUOsxxcpyFiIJ33xMdT9j
# 7CFfxCBRa2+xq4aLT8LWRV+dIPyhHsXAj6KxfgommfXkaS+YHS312amyHeUbAgMB
# AAGjggE6MIIBNjAPBgNVHRMBAf8EBTADAQH/MB0GA1UdDgQWBBTs1+OC0nFdZEzf
# Lmc/57qYrhwPTzAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzAOBgNV
# HQ8BAf8EBAMCAYYweQYIKwYBBQUHAQEEbTBrMCQGCCsGAQUFBzABhhhodHRwOi8v
# b2NzcC5kaWdpY2VydC5jb20wQwYIKwYBBQUHMAKGN2h0dHA6Ly9jYWNlcnRzLmRp
# Z2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcnQwRQYDVR0fBD4w
# PDA6oDigNoY0aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJl
# ZElEUm9vdENBLmNybDARBgNVHSAECjAIMAYGBFUdIAAwDQYJKoZIhvcNAQEMBQAD
# ggEBAHCgv0NcVec4X6CjdBs9thbX979XB72arKGHLOyFXqkauyL4hxppVCLtpIh3
# bb0aFPQTSnovLbc47/T/gLn4offyct4kvFIDyE7QKt76LVbP+fT3rDB6mouyXtTP
# 0UNEm0Mh65ZyoUi0mcudT6cGAxN3J0TU53/oWajwvy8LpunyNDzs9wPHh6jSTEAZ
# NUZqaVSwuKFWjuyk1T3osdz9HNj0d1pcVIxv76FQPfx2CWiEn2/K2yCNNWAcAgPL
# ILCsWKAOQGPFmCLBsln1VWvPJ6tsds5vIy30fnFqI2si/xK4VC0nftg62fC2h5b9
# W9FcrBjDTZ9ztwGpn1eqXijiuZQxggN2MIIDcgIBATB3MGMxCzAJBgNVBAYTAlVT
# MRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjE7MDkGA1UEAxMyRGlnaUNlcnQgVHJ1
# c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBUaW1lU3RhbXBpbmcgQ0ECEAVEr/OUnQg5
# pr/bP1/lYRYwDQYJYIZIAWUDBAIBBQCggdEwGgYJKoZIhvcNAQkDMQ0GCyqGSIb3
# DQEJEAEEMBwGCSqGSIb3DQEJBTEPFw0yNDAxMTYwMTM2NTBaMCsGCyqGSIb3DQEJ
# EAIMMRwwGjAYMBYEFGbwKzLCwskPgl3OqorJxk8ZnM9AMC8GCSqGSIb3DQEJBDEi
# BCDLdx3kBhSZdOASRaIz+AArS65IJ+vrbhhRfNvNTl3F8jA3BgsqhkiG9w0BCRAC
# LzEoMCYwJDAiBCDS9uRt7XQizNHUQFdoQTZvgoraVZquMxavTRqa1Ax4KDANBgkq
# hkiG9w0BAQEFAASCAgApXB0gJYIehw8Q6d+Vr4V5YHoJrG6UcqUD6UVgd1GWJzH2
# TIvoDPqQcXxRoT7MXNPojW60i/2Vxsf3vYhw+axYn97fpo4beIWn3JEmn+eLv1Fz
# q/5gqH3kjccr0qhKbdNKucdIfpA3S9xDHuNEdJvwSzPkKJbjiAMcSSgXYqSE4p6H
# y+1y78EGsLsszsjjJz/P3UXpXeD/qVVuEtGlh9w/KIaz6I1W3oAsyiTKjZ/E9qHf
# kzLrcowcEZu6QOoH1K5L5Ka1U3/B/Xpu5CiaQcg2ae95JDVXMhhAwHhWYjrEt1mT
# 3nC1ZpzbQ9fyeCYJKw+jrQye5QZr3gWiuc/NztAl1IACiI3gel5UtzM2SbpOuaaO
# i/8TyZmWP25zbV5rjER2Ojn8MnwkO976lv2do4JLVDbma5Mw7w9BK7SH7kkIjkNo
# x2ABUuf11FsDVypRnShwl3lQHXeQwo3DbbruHRIFp7xfL7axWmlyrxNXF0QrKtXP
# BiVpyJf8trDr4eJgxNpY3KtO6Poddlw/yUDfC8/F0ZTvLvt0tDwb8MGhOdCwsT41
# 8UvELasfp5do3MqnS+v//Tm+tTLKwVxFbr1wK/HVHmneSXZ7O/cmyOu9Np8me5XT
# pMe3/3CTpxaHb4StqZJaa/nZbZndAvMMQGqVHlt08mKdHEDM95JY3NLLyfmMeA==
# SIG # End signature block
