# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Amazon Software License (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
# http://aws.amazon.com/asl/
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

<#
    .SYNOPSIS
    Configures all required settings to initialize an EC2 instance.

    .DESCRIPTION
    During EC2 instance launch, this cmdlet goes through the following steps to configure all required settings and display information to the console.

    1. Wait for sysprep - Ensure that the sysprep process is finished before configuration begins.
    2. Add routes - Creates the routes for connecting to the Instance Metadata Service (IMDS), Windows activation servers, etc.
    3. Wait for metadata - Ensure that instance metadata is available.
    4. Rename computer - Rename the computer based on the instance IP address.
    5. Display instance info - Display information about the instance, and the AMI that it launched from.
    6. Extend boot volume - Extend the boot volume with unallocated space.
    7. Set password - Set the password, so that it is available from the console
    8. Windows is Ready - Display the following message in the console: "Message: Windows is Ready to use".
    9. Run user data - If there is a user data script in the instance metadata, run it. This step runs after displaying the 'Windows is Ready' message, to ensure that it doesn't hold up any other processes that depend on the message. This can be a long-running process.
    10. Register disabled scheduledTask - Keep the script as a scheduled task for future use.

    * The instance launch checks serial port setup by default.
    * If any task requires a reboot, the instance launch re-registers the script as scheduledTask.

    .PARAMETER Schedule
    Registers a scheduled task to run the script the next time the instance goes through the boot process.

    .PARAMETER SchedulePerBoot
    Registers a scheduled task to run the script every time the instance goes through the boot process.

    .PARAMETER KeepSchedule
    Keeps the scheduled task enabled after the script runs.

    .EXAMPLE
    ./InitializeInstance.ps1 -Schedule
#>

# Required for powershell to determine what parameter set to use when running with zero args (us a non existent set name)
[CmdletBinding(DefaultParameterSetName = 'Default')]
param(
  [Parameter(Mandatory = $false, ParameterSetName = "Schedule")]
  [switch]$Schedule = $false,

  [Parameter(Mandatory = $false, ParameterSetName = "SchedulePerBoot")]
  [switch]$SchedulePerBoot = $false,

  [Parameter(Mandatory = $false, ParameterSetName = "KeepSchedule")]
  [switch]$KeepSchedule = $false
)

Set-Variable rootPath -Option Constant -Scope Local -Value (Join-Path $env:ProgramData -ChildPath "Amazon\EC2-Windows\Launch")
Set-Variable modulePath -Option Constant -Scope Local -Value (Join-Path $rootPath -ChildPath "Module\Ec2Launch.psd1")
Set-Variable scriptPath -Option Constant -Scope Local -Value (Join-Path $PSScriptRoot -ChildPath $MyInvocation.MyCommand.Name)
Set-Variable telemetryLogFilePath -Option Constant -Scope Local -Value (Join-Path $rootPath -ChildPath "Log\Telemetry.log")
Set-Variable scheduleName -Option Constant -Scope Local -Value "Instance Initialization"

Set-Variable amazonSSMagent -Option Constant -Scope Local -Value "AmazonSSMAgent"
Set-Variable ssmAgentTimeoutSeconds -Option Constant -Scope Local -Value 300 # 5 minutes
Set-Variable powershellJobTimeoutSeconds -Option Constant -Scope Local -Value 120 # 2 minutes
Set-Variable ssmAgentSleepSeconds -Option Constant -Scope Local -Value 0.25 # 250 milliseconds

# Import Ec2Launch module to prepare to use helper functions.
Import-Module $modulePath

# Before calling any function, initialize the log with filename and also allow LogToConsole.
Initialize-Log -FileName "Ec2Launch.log" -AllowLogToConsole -RestrictToAdmins $false

# Before using the shared-library DLL provide it with agent, error, and console logging.
Open-LibraryLogging

if ($Schedule -or $SchedulePerBoot) {
  $arguments = $null
  if ($SchedulePerBoot) {
    # If you want to run on every reboot, use the '-KeepSchedule' argument to keep the schedule after InitializeInstance.ps1 runs.
    $arguments = "-KeepSchedule"

    # Disable user data schedule so that user data doesn't run twice (once on launch, and again in the external schedule).
    Invoke-Userdata -OnlyUnregister
  }

  # Scheduling script with no argument tells script to start normally.
  Register-ScriptScheduler -ScriptPath $scriptPath -ScheduleName $scheduleName -Arguments $arguments

  # Set AmazonSSMAgent StartupType to be Disabled to prevent AmazonSSMAgent from running util windows is ready.
  Set-Service $amazonSSMagent -StartupType Disabled -ErrorAction SilentlyContinue
  Write-Log "Instance initialization is scheduled successfully"
  Complete-Log
  exit 0
}

try {
  Write-Log "Initializing instance is started"

  # Serial Port must be available in your instance to send logs to console.
  # If serial port is not available, it sets the serial port and requests reboot.
  # If serial port is already available, it continues without reboot.
  if ((Test-NanoServer) -and (Set-SerialPort)) {
    # Now Computer can restart.
    Write-Log "Message: Windows is restarting..."
    Register-ScriptScheduler -ScriptPath $scriptPath -ScheduleName $scheduleName
    Restart-Computer
    exit 0
  }

  # Serial port COM1 must be opened before executing any task.
  Open-SerialPort
  Open-Telemetry -TelemetryLogFilePath $telemetryLogFilePath

  $isTelemetryFinalized = $false
  $finalizeTelemetry = {
    # Other logs, such as the user data log can be pushed onto the log stack. We want to write to the main agent log, so
    # it is pushed onto the stack here and we ensure that it is popped off the stack through the finally block.
    Initialize-Log -FileName "Ec2Launch.log" -AllowLogToConsole -RestrictToAdmins $false

    try {
      if ($script:isTelemetryFinalized) {
        Write-Log "Telemetry already finalized. Skipping finalize."
        return
      }

      Close-Telemetry

      $script:isTelemetryFinalized = $true
    }
    finally {
      Complete-Log
    }

  }.GetNewClosure()

  Send-TelemetryBool -FieldName "IsAgentScheduledPerBoot" -Value $KeepSchedule

  # Task must be executed after sysprep is complete.
  # WMI object seems to be missing during sysprep.
  Wait-Sysprep

  # Routes need to be added to connect to instance metadata service and KMS service.
  Add-Routes

  # Once routes are added, we need to wait for metadata to be available
  # becuase there are several tasks that need information from metadata.
  Wait-Metadata

  # Set KMS server and port in registry key.
  Set-ActivationSettings

  # Create wallpaper setup cmd file in windows startup directory, which
  # renders instance information on wallpaper as user logs in.
  New-WallpaperSetup

  # Installs EGPU for customers that request it
  Install-EgpuManager

  # Before renaming computer, it checks if computer is already renamed.
  # If computer is not renamed yet, it renames computer and requests reboot.
  # If computer is already renamed or failed to be renamed, it continues without reboot.
  if (Set-ComputerName) {
    # Now Computer can restart.
    Write-Log "Message: Windows is restarting..." -LogToConsole
    Register-ScriptScheduler -ScriptPath $scriptPath -ScheduleName $scheduleName
    & $finalizeTelemetry
    Restart-Computer
    exit 0
  }

  # All of the instance information is displayed to console.
  Send-AMIInfo
  Send-OSInfo
  Send-IDInfo
  Send-InstanceInfo
  Send-MsSqlInfo
  Send-DriverInfo
  Send-Ec2LaunchVersion
  Send-VSSVersion
  Send-SSMAgentVersion
  Send-RDPCertInfo
  Send-FeatureStatus

  # Add DNS suffixes in search list and store that in registry key.
  Add-DnsSuffixList

  # The volume size is extended with unallocated spaces.
  Set-BootVolumeSize

  # Disable Configure ENA Network settings. This setting has never been enabled on the agent.
  # Configure ENA Network settings
  # if (Set-ENAConfig) {
  #     # Now Computer can restart.
  #     Write-Log "Message: Windows is restarting..." -LogToConsole
  #     Register-ScriptScheduler -ScriptPath $scriptPath -ScheduleName $scheduleName
  #     Close-SerialPort
  #     Restart-Computer
  #     Exit 0
  # }

  # If requested, sets the monitor to never turn off which will interfere with acpi signals
  Set-MonitorAlwaysOn
  # If requested, tells windows to go in to hibernate instead of sleep
  # when the system sends the acpi sleep signal.
  Set-HibernateOnSleep

  # Password is randomly generated and provided to console in encrypted format.
  # Here, also admin account gets enabled.
  $adminCredential = Set-AdminAccount

  # Encrypt the admin credentials and send it to console.
  # Console understands the admin password and allows users to decrypt it with private key.
  if ($null -ne $adminCredential -and $adminCredential.UserName -and $adminCredential.GetNetworkCredential().Password) {
    Send-AdminCredential -AdminCredential $adminCredential
  }
  [System.GC]::Collect()

  try {
    # Set AmazonSSMAgent StartupType to be back to Automatic
    Set-Service $amazonSSMagent -StartupType Automatic -ErrorAction Stop
  }
  catch {
    Write-ErrorLog ("Failed to set AmazonSSMAgent service to Automatic {0}" -f $_.Exception.Message)
  }

  # Send the IsUserDataScheduledPerBoot telemetry metric.
  try {
    function SendIsUserDataScheduledPerBoot {
      $handleUserDataConfig = Get-LaunchConfig -Key HandleUserData
      [System.GC]::Collect()
      if (-not [System.Convert]::ToBoolean($handleUserDataConfig)) {
        return
      }

      $userdata = Get-Metadata -UrlFragment "user-data"
      if ([string]::IsNullOrWhiteSpace($userdata)) {
        return
      }

      $regexFormat = "(?is){0}(.*?){1}"
      $persistRegex = [regex]($regexFormat -f "<persist>","</persist>")
      $persistMatch = $persistRegex.Matches($userdata)
      if ((-not $persistMatch.Success) -or ($persistMatch.Captures.Count -ne 1) -or ($persistMatch.Groups.Count -ne 2)) {
        Send-TelemetryBool -FieldName "IsUserDataScheduledPerBoot" -Value $false
        return
      }

      $persistValue = $persistMatch.Groups[1].Value
      Send-TelemetryBool -FieldName "IsUserDataScheduledPerBoot" -Value ($persistValue -ieq "true")
    }

    SendIsUserDataScheduledPerBoot
  } catch {
    Write-ErrorLog ("Failed to parse the <persist> tag from user data: {0}" -f $_.Exception.Message)
  }

  # The instance console log is uploaded shortly after the "Windows is Ready" message is sent. To include
  # telemetry in that upload, telemetry is written before the "Windows is Ready" message is sent.
  & $finalizeTelemetry

  # Windows-is-ready message is displayed to console after all steps above are complete.
  Send-WindowsIsReady

  # Disable the scheduledTask if we were only suppose to run once, otherwise, leave the schedule.
  if (!$KeepSchedule) {
    Register-ScriptScheduler -ScriptPath $scriptPath -ScheduleName $scheduleName -Disabled
  }

  # Free the handle to the COM port before running user data.
  Close-SerialPort

  # If this run is from a "run on every boot" schedule, make sure we only execute user data (dont
  # schedule it as a separate task), this is so we can instead execute it inline on every boot.

  # User data scripts are optional. They can be long-running, and are not part of the windows-is-ready condition logic.
  # If a user data script exists, run it after the launch completes the Send-WindowsIsReady task.
  $persistUserData = Invoke-Userdata -AdminCredential $adminCredential -OnlyExecute:$KeepSchedule -FinalizeTelemetry $finalizeTelemetry

  try {
    # Start the AmazonSSMAgent Windows service.
    # Have to use closure argument list because the closure will be running in a sub-job that wont have access to local variables
    $startSsmAgentService = {
      $serviceName = $args[0]
      while ($true) {
        try {
          $service = Start-Service -Name $serviceName -PassThru -ErrorAction Stop
          if ($service.Status -eq "Running") {
            return
          }
        }
        catch {
          Write-Host ("Failed to start the '{0}' service: {1}" -f $serviceName, $_.Exception.Message)
        }
      }
    }
    $timeSinceStart = [System.Diagnostics.Stopwatch]::StartNew()
    while ("Running" -ne $(Get-Service -Name $amazonSSMagent).Status) {
      $totalSecondsSinceStart = $timeSinceStart.Elapsed.TotalSeconds
      if ($totalSecondsSinceStart -gt $ssmAgentTimeoutSeconds) {
        Write-ErrorLog "Timed out starting the AmazonSSMAgent Windows service."
        break
      }
      $jobTimeoutSeconds = [Math]::Min($powershellJobTimeoutSeconds, $ssmAgentTimeoutSeconds - $totalSecondsSinceStart)
      Invoke-WithTimeout -ScriptName $amazonSSMagent -ScriptBlock $startSsmAgentService -ArgumentList $amazonSSMagent -SleepSeconds $ssmAgentSleepSeconds -TimeoutSeconds $jobTimeoutSeconds
    }
  }
  catch {
    Write-ErrorLog ("Failed to start the AmazonSSMAgent Windows service: {0}" -f $_.Exception.Message)
  }

  # If this run is from a "run on every boot" schedule, disable certain functionality for future runs.
  if ($KeepSchedule) {
    Get-LaunchConfig -Key AdminPasswordType -Delete
    Get-LaunchConfig -Key SetMonitorAlwaysOn -Delete

    # Only disable handle user data if persist was false
    if (!$persistUserData) {
      Get-LaunchConfig -Key HandleUserData -Delete
    }
    [System.GC]::Collect()
  }

  Write-Log "Initializing instance is done"
  exit 0
}
catch {
  Write-ErrorLog ("Failed to continue initializing the instance: {0}" -f $_.Exception.Message)

  exit 1
}
finally {
  & $finalizeTelemetry

  # Free the handle to the COM port before finishing the script.
  Close-SerialPort

  # Complete the log before finishing the script.
  Complete-Log
}

# SIG # Begin signature block
# MIIueQYJKoZIhvcNAQcCoIIuajCCLmYCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCCUjomSeTBZMqFW
# PtV/g/XGo6LRxyRH5rvsYbrBIenOXaCCE+YwggXAMIIEqKADAgECAhAP0bvKeWvX
# +N1MguEKmpYxMA0GCSqGSIb3DQEBCwUAMGwxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xKzApBgNV
# BAMTIkRpZ2lDZXJ0IEhpZ2ggQXNzdXJhbmNlIEVWIFJvb3QgQ0EwHhcNMjIwMTEz
# MDAwMDAwWhcNMzExMTA5MjM1OTU5WjBiMQswCQYDVQQGEwJVUzEVMBMGA1UEChMM
# RGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSEwHwYDVQQD
# ExhEaWdpQ2VydCBUcnVzdGVkIFJvb3QgRzQwggIiMA0GCSqGSIb3DQEBAQUAA4IC
# DwAwggIKAoICAQC/5pBzaN675F1KPDAiMGkz7MKnJS7JIT3yithZwuEppz1Yq3aa
# za57G4QNxDAf8xukOBbrVsaXbR2rsnnyyhHS5F/WBTxSD1Ifxp4VpX6+n6lXFllV
# cq9ok3DCsrp1mWpzMpTREEQQLt+C8weE5nQ7bXHiLQwb7iDVySAdYyktzuxeTsiT
# +CFhmzTrBcZe7FsavOvJz82sNEBfsXpm7nfISKhmV1efVFiODCu3T6cw2Vbuyntd
# 463JT17lNecxy9qTXtyOj4DatpGYQJB5w3jHtrHEtWoYOAMQjdjUN6QuBX2I9YI+
# EJFwq1WCQTLX2wRzKm6RAXwhTNS8rhsDdV14Ztk6MUSaM0C/CNdaSaTC5qmgZ92k
# J7yhTzm1EVgX9yRcRo9k98FpiHaYdj1ZXUJ2h4mXaXpI8OCiEhtmmnTK3kse5w5j
# rubU75KSOp493ADkRSWJtppEGSt+wJS00mFt6zPZxd9LBADMfRyVw4/3IbKyEbe7
# f/LVjHAsQWCqsWMYRJUadmJ+9oCw++hkpjPRiQfhvbfmQ6QYuKZ3AeEPlAwhHbJU
# KSWJbOUOUlFHdL4mrLZBdd56rF+NP8m800ERElvlEFDrMcXKchYiCd98THU/Y+wh
# X8QgUWtvsauGi0/C1kVfnSD8oR7FwI+isX4KJpn15GkvmB0t9dmpsh3lGwIDAQAB
# o4IBZjCCAWIwDwYDVR0TAQH/BAUwAwEB/zAdBgNVHQ4EFgQU7NfjgtJxXWRM3y5n
# P+e6mK4cD08wHwYDVR0jBBgwFoAUsT7DaQP4v0cB1JgmGggC72NkK8MwDgYDVR0P
# AQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMH8GCCsGAQUFBwEBBHMwcTAk
# BggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEkGCCsGAQUFBzAC
# hj1odHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRIaWdoQXNzdXJh
# bmNlRVZSb290Q0EuY3J0MEsGA1UdHwREMEIwQKA+oDyGOmh0dHA6Ly9jcmwzLmRp
# Z2ljZXJ0LmNvbS9EaWdpQ2VydEhpZ2hBc3N1cmFuY2VFVlJvb3RDQS5jcmwwHAYD
# VR0gBBUwEzAHBgVngQwBAzAIBgZngQwBBAEwDQYJKoZIhvcNAQELBQADggEBAEHx
# qRH0DxNHecllao3A7pgEpMbjDPKisedfYk/ak1k2zfIe4R7sD+EbP5HU5A/C5pg0
# /xkPZigfT2IxpCrhKhO61z7H0ZL+q93fqpgzRh9Onr3g7QdG64AupP2uU7SkwaT1
# IY1rzAGt9Rnu15ClMlIr28xzDxj4+87eg3Gn77tRWwR2L62t0+od/P1Tk+WMieNg
# GbngLyOOLFxJy34riDkruQZhiPOuAnZ2dMFkkbiJUZflhX0901emWG4f7vtpYeJa
# 3Cgh6GO6Ps9W7Zrk9wXqyvPsEt84zdp7PiuTUy9cUQBY3pBIowrHC/Q7bVUx8ALM
# R3eWUaNetbxcyEMRoacwggawMIIEmKADAgECAhAIrUCyYNKcTJ9ezam9k67ZMA0G
# CSqGSIb3DQEBDAUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJ
# bmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0
# IFRydXN0ZWQgUm9vdCBHNDAeFw0yMTA0MjkwMDAwMDBaFw0zNjA0MjgyMzU5NTla
# MGkxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UE
# AxM4RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEz
# ODQgMjAyMSBDQTEwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDVtC9C
# 0CiteLdd1TlZG7GIQvUzjOs9gZdwxbvEhSYwn6SOaNhc9es0JAfhS0/TeEP0F9ce
# 2vnS1WcaUk8OoVf8iJnBkcyBAz5NcCRks43iCH00fUyAVxJrQ5qZ8sU7H/Lvy0da
# E6ZMswEgJfMQ04uy+wjwiuCdCcBlp/qYgEk1hz1RGeiQIXhFLqGfLOEYwhrMxe6T
# SXBCMo/7xuoc82VokaJNTIIRSFJo3hC9FFdd6BgTZcV/sk+FLEikVoQ11vkunKoA
# FdE3/hoGlMJ8yOobMubKwvSnowMOdKWvObarYBLj6Na59zHh3K3kGKDYwSNHR7Oh
# D26jq22YBoMbt2pnLdK9RBqSEIGPsDsJ18ebMlrC/2pgVItJwZPt4bRc4G/rJvmM
# 1bL5OBDm6s6R9b7T+2+TYTRcvJNFKIM2KmYoX7BzzosmJQayg9Rc9hUZTO1i4F4z
# 8ujo7AqnsAMrkbI2eb73rQgedaZlzLvjSFDzd5Ea/ttQokbIYViY9XwCFjyDKK05
# huzUtw1T0PhH5nUwjewwk3YUpltLXXRhTT8SkXbev1jLchApQfDVxW0mdmgRQRNY
# mtwmKwH0iU1Z23jPgUo+QEdfyYFQc4UQIyFZYIpkVMHMIRroOBl8ZhzNeDhFMJlP
# /2NPTLuqDQhTQXxYPUez+rbsjDIJAsxsPAxWEQIDAQABo4IBWTCCAVUwEgYDVR0T
# AQH/BAgwBgEB/wIBADAdBgNVHQ4EFgQUaDfg67Y7+F8Rhvv+YXsIiGX0TkIwHwYD
# VR0jBBgwFoAU7NfjgtJxXWRM3y5nP+e6mK4cD08wDgYDVR0PAQH/BAQDAgGGMBMG
# A1UdJQQMMAoGCCsGAQUFBwMDMHcGCCsGAQUFBwEBBGswaTAkBggrBgEFBQcwAYYY
# aHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEEGCCsGAQUFBzAChjVodHRwOi8vY2Fj
# ZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNydDBDBgNV
# HR8EPDA6MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRU
# cnVzdGVkUm9vdEc0LmNybDAcBgNVHSAEFTATMAcGBWeBDAEDMAgGBmeBDAEEATAN
# BgkqhkiG9w0BAQwFAAOCAgEAOiNEPY0Idu6PvDqZ01bgAhql+Eg08yy25nRm95Ry
# sQDKr2wwJxMSnpBEn0v9nqN8JtU3vDpdSG2V1T9J9Ce7FoFFUP2cvbaF4HZ+N3HL
# IvdaqpDP9ZNq4+sg0dVQeYiaiorBtr2hSBh+3NiAGhEZGM1hmYFW9snjdufE5Btf
# Q/g+lP92OT2e1JnPSt0o618moZVYSNUa/tcnP/2Q0XaG3RywYFzzDaju4ImhvTnh
# OE7abrs2nfvlIVNaw8rpavGiPttDuDPITzgUkpn13c5UbdldAhQfQDN8A+KVssIh
# dXNSy0bYxDQcoqVLjc1vdjcshT8azibpGL6QB7BDf5WIIIJw8MzK7/0pNVwfiThV
# 9zeKiwmhywvpMRr/LhlcOXHhvpynCgbWJme3kuZOX956rEnPLqR0kq3bPKSchh/j
# wVYbKyP/j7XqiHtwa+aguv06P0WmxOgWkVKLQcBIhEuWTatEQOON8BUozu3xGFYH
# Ki8QxAwIZDwzj64ojDzLj4gLDb879M4ee47vtevLt/B3E+bnKD+sEq6lLyJsQfmC
# XBVmzGwOysWGw/YmMwwHS6DTBwJqakAwSEs0qFEgu60bhQjiWQ1tygVQK+pKHJ6l
# /aCnHwZ05/LWUpD9r4VIIflXO7ScA+2GRfS0YW6/aOImYIbqyK+p/pQd52MbOoZW
# eE4wggdqMIIFUqADAgECAhAPo79ZA+boHMkRkUNkUoBpMA0GCSqGSIb3DQEBCwUA
# MGkxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UE
# AxM4RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEz
# ODQgMjAyMSBDQTEwHhcNMjMwOTIxMDAwMDAwWhcNMjQwOTIwMjM1OTU5WjCB8jET
# MBEGCysGAQQBgjc8AgEDEwJVUzEZMBcGCysGAQQBgjc8AgECEwhEZWxhd2FyZTEd
# MBsGA1UEDwwUUHJpdmF0ZSBPcmdhbml6YXRpb24xEDAOBgNVBAUTBzQxNTI5NTQx
# CzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdTZWF0
# dGxlMSIwIAYDVQQKExlBbWF6b24gV2ViIFNlcnZpY2VzLCBJbmMuMRMwEQYDVQQL
# EwpBbWF6b24gRUMyMSIwIAYDVQQDExlBbWF6b24gV2ViIFNlcnZpY2VzLCBJbmMu
# MIIBojANBgkqhkiG9w0BAQEFAAOCAY8AMIIBigKCAYEAyQgSqeEEf92N/lmb69Tz
# t7Qg0dCGMeN5rpoKPFsW6OpN0p/9gNqeqHTGjeKITMHC2F2OkQ+adZfFIniOKVuB
# kzAgr1JvLGP7nGCB1qfU0DowEG5YOQYxq0+ez8ARXA5vWO2DkJyJ72upyEBV+Uuc
# yffwPV8iino2w2JcKBgSBwrfK09odd3TvacQrGZflbOvBZkRwzpgbcqKadU+xWlc
# absOS/jZhk+lXpEFPx/6U2PA8F+FpTFHqrrsB8l7LHqqdPHfB6CwszM16KnpWc07
# vI7ESveeKHWEXra3Fg70XmOby8bUTlW9U2IfotsH4yxWy/odA1nFBZ7ihXFN+IhK
# ABRFlAMHLjhd/ZzmexZH0ZAfPSrPUwfeMAJNKX8lHnuz5gQT0ukhhXyyp+JY2X9q
# ii6BopteCUYMEj/4KPQuTHbYSK+HVDk8yUpt7ZnFkihJ4eAQ+doHuMPufJ+rnxOs
# a6UahM1ts+ili7Nym8j1cuzQjfrjjysjRxqUGI1+9FVFAgMBAAGjggICMIIB/jAf
# BgNVHSMEGDAWgBRoN+Drtjv4XxGG+/5hewiIZfROQjAdBgNVHQ4EFgQUN8tpbyMu
# Fv5oH1HZXuepuLWCBrEwPQYDVR0gBDYwNDAyBgVngQwBAzApMCcGCCsGAQUFBwIB
# FhtodHRwOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMwDgYDVR0PAQH/BAQDAgeAMBMG
# A1UdJQQMMAoGCCsGAQUFBwMDMIG1BgNVHR8Ega0wgaowU6BRoE+GTWh0dHA6Ly9j
# cmwzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNENvZGVTaWduaW5nUlNB
# NDA5NlNIQTM4NDIwMjFDQTEuY3JsMFOgUaBPhk1odHRwOi8vY3JsNC5kaWdpY2Vy
# dC5jb20vRGlnaUNlcnRUcnVzdGVkRzRDb2RlU2lnbmluZ1JTQTQwOTZTSEEzODQy
# MDIxQ0ExLmNybDCBlAYIKwYBBQUHAQEEgYcwgYQwJAYIKwYBBQUHMAGGGGh0dHA6
# Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBcBggrBgEFBQcwAoZQaHR0cDovL2NhY2VydHMu
# ZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZEc0Q29kZVNpZ25pbmdSU0E0MDk2
# U0hBMzg0MjAyMUNBMS5jcnQwCQYDVR0TBAIwADANBgkqhkiG9w0BAQsFAAOCAgEA
# WtvDAAUi8Az7ODQJNSv4R7zXjN8JRghkMSQHD8CIcAuLER+B7zfbHLqwU8sdzxQs
# VkMGAPlwrcybnqFipQgOpz3MUHiAseVzWRz/7RIkmZDL6c6X4wJka1si5P+XxsC1
# KGFbYgIu+n4sOf00AOHuSYYtiE85K0JbClplz2Oc1E8sb4V2Xpxg7vzPLuCInD84
# E6TFcmIZv32RiLcgWNv+lXlWr4KCu4kvqKWuB98zxlyIVvUTyGRJMBeOKYXCxPUN
# xvVQ33g2fl1YYmCAl4u+0Thlw/LnibUNsUO08uKYqqynCjw7GNA9q0syTOe1eB6a
# d8y3E6zA2kVVRGouj43MWdniAheQ9HUsSQ8uNK6IrhPDU75DxB+UTGCAc4Y8ZYbE
# 3OtazqEMzH3kBU9o2JLZ/EE+0Cs0qlFT0xFQvEGn8VnEaQ+V0RfUwU8XJQUbkh20
# edcdODciyg3BUTqP18IcW16lS7Bvjmy/I+9mHwDZKN5Lxs42Rs4+NY7YxK+Uau8F
# lx7s1jibMa0g15HJFvmEO4BB9r4jozQykGb30E7smq+lvsMFLlv4L956TXfJngZI
# 3P7HAaUORR+nieq2ujgqpBiAwccIcK+YtRsoCBNp0C0AMdWEeSVLf6mAE61JVxju
# N+FxDlvDx5Y0tszY58Uw+N4jwVT8Z1D6q1lG5nmYPsYxghnpMIIZ5QIBATB9MGkx
# CzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UEAxM4
# RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEzODQg
# MjAyMSBDQTECEA+jv1kD5ugcyRGRQ2RSgGkwDQYJYIZIAWUDBAIBBQCgfDAQBgor
# BgEEAYI3AgEMMQIwADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEE
# AYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0BCQQxIgQgtEu8f+WcZ0wu
# i4an6oggCcrQVqlQAJC6d2NmmmRl3YEwDQYJKoZIhvcNAQEBBQAEggGAVZbYKYRy
# FESW3UeCslRigdXTbE1EwuXklPrgQQupyarI607HWBYQ5h1Ft9pndPXpl31W651F
# RsF7TR7WbeElS7ZlbpxaZQP7kTBgGKVHlXeO3QKk4yPaukazhhwdiU2YZoExtI8B
# 37sjhHNHrM9eetu7BiP6ivz4P7rEk1UbjA2xwObu/yyH3/eDF+Y/mhvSkI+AjSvE
# jGzLuLPtT2mZdJg1E2LyZFp7BFa5K+OFrlNYmj5ddjEVDXEKQhVENr19bqzS+uND
# rs1THz6gVGGZLi3MrzqMgbaEBD8PL6+0U7BDiQSngjzAQQdlpuIKb7lHQMjnwzUp
# 3ZMNPizfDmrgwvd8s20E/wl8jcBZH0OcgPEQi0tvQpUAdqqrHZaqxY1TOpiTroid
# 6OVa1mo/hjn7bvWqgGdEMde5liCyzYoqXlV50yDuRQ3RJV79YsorT2j7UpSowLBU
# 2QiAxPLlRWfJdaCaUPI+otFDr1Rahg/h/OG13wxsMx95feNwKRHFyKBUoYIXPzCC
# FzsGCisGAQQBgjcDAwExghcrMIIXJwYJKoZIhvcNAQcCoIIXGDCCFxQCAQMxDzAN
# BglghkgBZQMEAgEFADB3BgsqhkiG9w0BCRABBKBoBGYwZAIBAQYJYIZIAYb9bAcB
# MDEwDQYJYIZIAWUDBAIBBQAEIE059/99V5ZODqjxNZsEO58vPKYnHNO+cpY6/OBZ
# ZIehAhBGfN57G+HVnsTBjK7DL0AMGA8yMDI0MDExNjAxNDM0OFqgghMJMIIGwjCC
# BKqgAwIBAgIQBUSv85SdCDmmv9s/X+VhFjANBgkqhkiG9w0BAQsFADBjMQswCQYD
# VQQGEwJVUzEXMBUGA1UEChMORGlnaUNlcnQsIEluYy4xOzA5BgNVBAMTMkRpZ2lD
# ZXJ0IFRydXN0ZWQgRzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0YW1waW5nIENBMB4X
# DTIzMDcxNDAwMDAwMFoXDTM0MTAxMzIzNTk1OVowSDELMAkGA1UEBhMCVVMxFzAV
# BgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMSAwHgYDVQQDExdEaWdpQ2VydCBUaW1lc3Rh
# bXAgMjAyMzCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAKNTRYcdg45b
# rD5UsyPgz5/X5dLnXaEOCdwvSKOXejsqnGfcYhVYwamTEafNqrJq3RApih5iY2nT
# WJw1cb86l+uUUI8cIOrHmjsvlmbjaedp/lvD1isgHMGXlLSlUIHyz8sHpjBoyoNC
# 2vx/CSSUpIIa2mq62DvKXd4ZGIX7ReoNYWyd/nFexAaaPPDFLnkPG2ZS48jWPl/a
# Q9OE9dDH9kgtXkV1lnX+3RChG4PBuOZSlbVH13gpOWvgeFmX40QrStWVzu8IF+qC
# ZE3/I+PKhu60pCFkcOvV5aDaY7Mu6QXuqvYk9R28mxyyt1/f8O52fTGZZUdVnUok
# L6wrl76f5P17cz4y7lI0+9S769SgLDSb495uZBkHNwGRDxy1Uc2qTGaDiGhiu7xB
# G3gZbeTZD+BYQfvYsSzhUa+0rRUGFOpiCBPTaR58ZE2dD9/O0V6MqqtQFcmzyrzX
# xDtoRKOlO0L9c33u3Qr/eTQQfqZcClhMAD6FaXXHg2TWdc2PEnZWpST618RrIbro
# HzSYLzrqawGw9/sqhux7UjipmAmhcbJsca8+uG+W1eEQE/5hRwqM/vC2x9XH3mwk
# 8L9CgsqgcT2ckpMEtGlwJw1Pt7U20clfCKRwo+wK8REuZODLIivK8SgTIUlRfgZm
# 0zu++uuRONhRB8qUt+JQofM604qDy0B7AgMBAAGjggGLMIIBhzAOBgNVHQ8BAf8E
# BAMCB4AwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDAgBgNV
# HSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEwHwYDVR0jBBgwFoAUuhbZbU2F
# L3MpdpovdYxqII+eyG8wHQYDVR0OBBYEFKW27xPn783QZKHVVqllMaPe1eNJMFoG
# A1UdHwRTMFEwT6BNoEuGSWh0dHA6Ly9jcmwzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2Vy
# dFRydXN0ZWRHNFJTQTQwOTZTSEEyNTZUaW1lU3RhbXBpbmdDQS5jcmwwgZAGCCsG
# AQUFBwEBBIGDMIGAMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5j
# b20wWAYIKwYBBQUHMAKGTGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdp
# Q2VydFRydXN0ZWRHNFJTQTQwOTZTSEEyNTZUaW1lU3RhbXBpbmdDQS5jcnQwDQYJ
# KoZIhvcNAQELBQADggIBAIEa1t6gqbWYF7xwjU+KPGic2CX/yyzkzepdIpLsjCIC
# qbjPgKjZ5+PF7SaCinEvGN1Ott5s1+FgnCvt7T1IjrhrunxdvcJhN2hJd6PrkKoS
# 1yeF844ektrCQDifXcigLiV4JZ0qBXqEKZi2V3mP2yZWK7Dzp703DNiYdk9WuVLC
# tp04qYHnbUFcjGnRuSvExnvPnPp44pMadqJpddNQ5EQSviANnqlE0PjlSXcIWiHF
# tM+YlRpUurm8wWkZus8W8oM3NG6wQSbd3lqXTzON1I13fXVFoaVYJmoDRd7ZULVQ
# jK9WvUzF4UbFKNOt50MAcN7MmJ4ZiQPq1JE3701S88lgIcRWR+3aEUuMMsOI5lji
# tts++V+wQtaP4xeR0arAVeOGv6wnLEHQmjNKqDbUuXKWfpd5OEhfysLcPTLfddY2
# Z1qJ+Panx+VPNTwAvb6cKmx5AdzaROY63jg7B145WPR8czFVoIARyxQMfq68/qTr
# eWWqaNYiyjvrmoI1VygWy2nyMpqy0tg6uLFGhmu6F/3Ed2wVbK6rr3M66ElGt9V/
# zLY4wNjsHPW2obhDLN9OTH0eaHDAdwrUAuBcYLso/zjlUlrWrBciI0707NMX+1Br
# /wd3H3GXREHJuEbTbDJ8WC9nR2XlG3O2mflrLAZG70Ee8PBf4NvZrZCARK+AEEGK
# MIIGrjCCBJagAwIBAgIQBzY3tyRUfNhHrP0oZipeWzANBgkqhkiG9w0BAQsFADBi
# MQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3
# d3cuZGlnaWNlcnQuY29tMSEwHwYDVQQDExhEaWdpQ2VydCBUcnVzdGVkIFJvb3Qg
# RzQwHhcNMjIwMzIzMDAwMDAwWhcNMzcwMzIyMjM1OTU5WjBjMQswCQYDVQQGEwJV
# UzEXMBUGA1UEChMORGlnaUNlcnQsIEluYy4xOzA5BgNVBAMTMkRpZ2lDZXJ0IFRy
# dXN0ZWQgRzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0YW1waW5nIENBMIICIjANBgkq
# hkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAxoY1BkmzwT1ySVFVxyUDxPKRN6mXUaHW
# 0oPRnkyibaCwzIP5WvYRoUQVQl+kiPNo+n3znIkLf50fng8zH1ATCyZzlm34V6gC
# ff1DtITaEfFzsbPuK4CEiiIY3+vaPcQXf6sZKz5C3GeO6lE98NZW1OcoLevTsbV1
# 5x8GZY2UKdPZ7Gnf2ZCHRgB720RBidx8ald68Dd5n12sy+iEZLRS8nZH92GDGd1f
# tFQLIWhuNyG7QKxfst5Kfc71ORJn7w6lY2zkpsUdzTYNXNXmG6jBZHRAp8ByxbpO
# H7G1WE15/tePc5OsLDnipUjW8LAxE6lXKZYnLvWHpo9OdhVVJnCYJn+gGkcgQ+ND
# Y4B7dW4nJZCYOjgRs/b2nuY7W+yB3iIU2YIqx5K/oN7jPqJz+ucfWmyU8lKVEStY
# dEAoq3NDzt9KoRxrOMUp88qqlnNCaJ+2RrOdOqPVA+C/8KI8ykLcGEh/FDTP0kyr
# 75s9/g64ZCr6dSgkQe1CvwWcZklSUPRR8zZJTYsg0ixXNXkrqPNFYLwjjVj33GHe
# k/45wPmyMKVM1+mYSlg+0wOI/rOP015LdhJRk8mMDDtbiiKowSYI+RQQEgN9XyO7
# ZONj4KbhPvbCdLI/Hgl27KtdRnXiYKNYCQEoAA6EVO7O6V3IXjASvUaetdN2udIO
# a5kM0jO0zbECAwEAAaOCAV0wggFZMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0O
# BBYEFLoW2W1NhS9zKXaaL3WMaiCPnshvMB8GA1UdIwQYMBaAFOzX44LScV1kTN8u
# Zz/nupiuHA9PMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDCDB3
# BggrBgEFBQcBAQRrMGkwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0
# LmNvbTBBBggrBgEFBQcwAoY1aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0Rp
# Z2lDZXJ0VHJ1c3RlZFJvb3RHNC5jcnQwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDov
# L2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZFJvb3RHNC5jcmwwIAYD
# VR0gBBkwFzAIBgZngQwBBAIwCwYJYIZIAYb9bAcBMA0GCSqGSIb3DQEBCwUAA4IC
# AQB9WY7Ak7ZvmKlEIgF+ZtbYIULhsBguEE0TzzBTzr8Y+8dQXeJLKftwig2qKWn8
# acHPHQfpPmDI2AvlXFvXbYf6hCAlNDFnzbYSlm/EUExiHQwIgqgWvalWzxVzjQEi
# Jc6VaT9Hd/tydBTX/6tPiix6q4XNQ1/tYLaqT5Fmniye4Iqs5f2MvGQmh2ySvZ18
# 0HAKfO+ovHVPulr3qRCyXen/KFSJ8NWKcXZl2szwcqMj+sAngkSumScbqyQeJsG3
# 3irr9p6xeZmBo1aGqwpFyd/EjaDnmPv7pp1yr8THwcFqcdnGE4AJxLafzYeHJLtP
# o0m5d2aR8XKc6UsCUqc3fpNTrDsdCEkPlM05et3/JWOZJyw9P2un8WbDQc1PtkCb
# ISFA0LcTJM3cHXg65J6t5TRxktcma+Q4c6umAU+9Pzt4rUyt+8SVe+0KXzM5h0F4
# ejjpnOHdI/0dKNPH+ejxmF/7K9h+8kaddSweJywm228Vex4Ziza4k9Tm8heZWcpw
# 8De/mADfIBZPJ/tgZxahZrrdVcA6KYawmKAr7ZVBtzrVFZgxtGIJDwq9gdkT/r+k
# 0fNX2bwE+oLeMt8EifAAzV3C+dAjfwAL5HYCJtnwZXZCpimHCUcr5n8apIUP/JiW
# 9lVUKx+A+sDyDivl1vupL0QVSucTDh3bNzgaoSv27dZ8/DCCBY0wggR1oAMCAQIC
# EA6bGI750C3n79tQ4ghAGFowDQYJKoZIhvcNAQEMBQAwZTELMAkGA1UEBhMCVVMx
# FTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNv
# bTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTIyMDgw
# MTAwMDAwMFoXDTMxMTEwOTIzNTk1OVowYjELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UE
# AxMYRGlnaUNlcnQgVHJ1c3RlZCBSb290IEc0MIICIjANBgkqhkiG9w0BAQEFAAOC
# Ag8AMIICCgKCAgEAv+aQc2jeu+RdSjwwIjBpM+zCpyUuySE98orYWcLhKac9WKt2
# ms2uexuEDcQwH/MbpDgW61bGl20dq7J58soR0uRf1gU8Ug9SH8aeFaV+vp+pVxZZ
# VXKvaJNwwrK6dZlqczKU0RBEEC7fgvMHhOZ0O21x4i0MG+4g1ckgHWMpLc7sXk7I
# k/ghYZs06wXGXuxbGrzryc/NrDRAX7F6Zu53yEioZldXn1RYjgwrt0+nMNlW7sp7
# XeOtyU9e5TXnMcvak17cjo+A2raRmECQecN4x7axxLVqGDgDEI3Y1DekLgV9iPWC
# PhCRcKtVgkEy19sEcypukQF8IUzUvK4bA3VdeGbZOjFEmjNAvwjXWkmkwuapoGfd
# pCe8oU85tRFYF/ckXEaPZPfBaYh2mHY9WV1CdoeJl2l6SPDgohIbZpp0yt5LHucO
# Y67m1O+SkjqePdwA5EUlibaaRBkrfsCUtNJhbesz2cXfSwQAzH0clcOP9yGyshG3
# u3/y1YxwLEFgqrFjGESVGnZifvaAsPvoZKYz0YkH4b235kOkGLimdwHhD5QMIR2y
# VCkliWzlDlJRR3S+Jqy2QXXeeqxfjT/JvNNBERJb5RBQ6zHFynIWIgnffEx1P2Ps
# IV/EIFFrb7GrhotPwtZFX50g/KEexcCPorF+CiaZ9eRpL5gdLfXZqbId5RsCAwEA
# AaOCATowggE2MA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0OBBYEFOzX44LScV1kTN8u
# Zz/nupiuHA9PMB8GA1UdIwQYMBaAFEXroq/0ksuCMS1Ri6enIZ3zbcgPMA4GA1Ud
# DwEB/wQEAwIBhjB5BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9v
# Y3NwLmRpZ2ljZXJ0LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNydDBFBgNVHR8EPjA8
# MDqgOKA2hjRodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVk
# SURSb290Q0EuY3JsMBEGA1UdIAQKMAgwBgYEVR0gADANBgkqhkiG9w0BAQwFAAOC
# AQEAcKC/Q1xV5zhfoKN0Gz22Ftf3v1cHvZqsoYcs7IVeqRq7IviHGmlUIu2kiHdt
# vRoU9BNKei8ttzjv9P+Aufih9/Jy3iS8UgPITtAq3votVs/59PesMHqai7Je1M/R
# Q0SbQyHrlnKhSLSZy51PpwYDE3cnRNTnf+hZqPC/Lwum6fI0POz3A8eHqNJMQBk1
# RmppVLC4oVaO7KTVPeix3P0c2PR3WlxUjG/voVA9/HYJaISfb8rbII01YBwCA8sg
# sKxYoA5AY8WYIsGyWfVVa88nq2x2zm8jLfR+cWojayL/ErhULSd+2DrZ8LaHlv1b
# 0VysGMNNn3O3AamfV6peKOK5lDGCA3YwggNyAgEBMHcwYzELMAkGA1UEBhMCVVMx
# FzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMTswOQYDVQQDEzJEaWdpQ2VydCBUcnVz
# dGVkIEc0IFJTQTQwOTYgU0hBMjU2IFRpbWVTdGFtcGluZyBDQQIQBUSv85SdCDmm
# v9s/X+VhFjANBglghkgBZQMEAgEFAKCB0TAaBgkqhkiG9w0BCQMxDQYLKoZIhvcN
# AQkQAQQwHAYJKoZIhvcNAQkFMQ8XDTI0MDExNjAxNDM0OFowKwYLKoZIhvcNAQkQ
# AgwxHDAaMBgwFgQUZvArMsLCyQ+CXc6qisnGTxmcz0AwLwYJKoZIhvcNAQkEMSIE
# IB5NbIjl+qWT89sLbqNEsXrlWwep0bkVb8W1V75s1NtmMDcGCyqGSIb3DQEJEAIv
# MSgwJjAkMCIEINL25G3tdCLM0dRAV2hBNm+CitpVmq4zFq9NGprUDHgoMA0GCSqG
# SIb3DQEBAQUABIICAHHKhvRmcxyzEH/q+zKZHxlt5gAZt6MaAu04Y833r5W2IutS
# +VjuFXXt4kJXvqAPiLAC/o8yRn1MEkEmx8uWgSNotryMMw7BnLtACbqN6DCqpMbA
# F3bwKQ4uxcB+r2GM4zcRyiX6wbiqu/l2vC7HAStWeR/vtHQT+hIzYKYT0cDWe9e5
# UI6onlzKcFrMFRReQKFwnYGB1BVB1481xxt1L7chUr+hSGIdWCWddq5zFYrh2ZWq
# LCJmtuy6Bk18xsIyHRI7PPawF+TwjVzk7EWII7kXAAwNmZgKeIrPg9sCEkFhBn4f
# 3jK6M4PePzizjXdyx0mc9BOxI/BGfaH/IJwtC1iZBccPu1ChHWV/i5Wbkci/KLhV
# nuq8UcdZLkkMTv2HK4Px4XGbRCuMnPNJJ814oMxJcAQshV/x1LsVVItXPfqn/0iE
# +U8McUigf+2CTvRQpwhZYF+Uybs3RNtpPrw+sYpTpYpHz4pSamdcqrjOBurof86X
# vhhQ+Qsr3BKSfGzB7cYcFRjj4vWiFRmX+iFe66HPFV7VG+2MvwFcGjNqReDO6QlY
# uFilUGzeA1jw8AIA1nL8B8XAYJpcGvmwCpC8zz02rx/1gu3VxCDFknAkR/SRy5PQ
# a3K6kNRRnJqPFYumnyG7zVE4OqYeSwNT4o/42ZVSs04tRolrxMbw1uB6ZRZ3
# SIG # End signature block
